import os
from PIL import Image
import imagehash
import threading
import queue

Image.MAX_IMAGE_PIXELS = 200000000  # Allow images up to 200 million pixels

def process_image(q, folder_path, hashes, duplicate_images, destination_folder, hash_size, cutoff, lock):
    while True:
        try:
            filename = q.get_nowait()
        except queue.Empty:
            break  # If the queue is empty, exit the thread
            
        print(f"Processing image: {filename}")
        image_path = os.path.join(folder_path, filename)

        try:
            # Calculate the hash of the image
            with Image.open(image_path) as img:
                temp_hash = imagehash.average_hash(img, hash_size)
            
            with lock:
                # Check if the hash already exists in the dictionary
                if temp_hash in hashes:
                    print(f"Duplicate found: {filename} and {hashes[temp_hash]}")
                    duplicate_images.append((filename, hashes[temp_hash]))
                    os.rename(image_path, os.path.join(destination_folder, filename))
                else:
                    # Check for near-duplicates
                    for h, existing_file in hashes.items():
                        if abs(temp_hash - h) < cutoff:
                            print(f"Near-duplicate found: {filename} and {existing_file}")
                            duplicate_images.append((filename, existing_file))
                            os.rename(image_path, os.path.join(destination_folder, filename))
                            break
                    else:
                        hashes[temp_hash] = filename
        except Exception as e:
            print(f"Error processing {filename}: {e}")

        q.task_done()

def find_duplicates_multithreaded(folder_path, destination_folder, hash_size=8, cutoff=2, num_threads=4):
    if not os.path.exists(destination_folder):
        os.makedirs(destination_folder)

    q = queue.Queue()
    lock = threading.Lock()

    hashes = {}
    duplicate_images = []

    for filename in os.listdir(folder_path):
        if filename.lower().endswith(('.png', '.jpg', '.jpeg', '.gif', '.bmp')):
            q.put(filename)

    threads = []
    for _ in range(num_threads):
        t = threading.Thread(target=process_image, args=(q, folder_path, hashes, duplicate_images, destination_folder, hash_size, cutoff, lock))
        t.start()
        threads.append(t)

    for t in threads:
        t.join()

    return duplicate_images

# Set the folder path and destination folder path
source_folder = r'C:\Users\King\Music\a'
dest_folder = r'C:\Users\King\Music\a\dupes'

# Run the duplicate finder
duplicates = find_duplicates_multithreaded(source_folder, dest_folder)

# Save the list of duplicates to a file
with open('duplicates.txt', 'w') as f:
    for dup in duplicates:
        f.write(f"{dup[0]} is a duplicate/near-duplicate of {dup[1]}\n")

print(f"Found {len(duplicates)} duplicates/near-duplicates. Results saved to duplicates.txt.")
